/**
 * @file
 * Provides utilities for native browser CSS Grid layout.
 */

(function (Drupal, _db) {

  'use strict';

  Drupal.gridstack = Drupal.gridstack || {};

  /**
   * GridStack native front-end public methods.
   *
   * @namespace
   */
  Drupal.gridstack.native = _db.extend({}, Drupal.gridstack.fe || {}, {

    onClass: 'gn',

    fixAspectRatio: function (el, e) {
      var me = this;
      var ch = me.options.cellHeight;
      var ok = ch === 'auto' || (me.cellHeight >= (ch / 2) && me.cellHeight < (ch * 2));
      var vm = me.vm > 0 ? (me.vm / 2) : 0;

      if (me.isEnabled(el)) {
        if (me.vm > 0) {
          el.style.gap = me.vm + 'px';
        }

        // Best if the gap or me.vm = 0.
        // @todo bad: var chvm = +(Math.round((me.cellHeight - vm) / 10).toFixed(2));
        var chvm = (me.cellHeight - vm) / 10;

        // Prevents weird calc during resizing.
        chvm = typeof e === 'undefined' ? chvm : (me.cellHeight / 10);

        // The magic .5, surprisingly making perfect square with gap 0.
        var magic = me.vm > 0 ? 1 : .5;
        me.cellHeight = ok ? (chvm * 10) + magic : ch;
        el.style.gridAutoRows = me.cellHeight + 'px';
      }
    },

    postUpdateGrid: function (el, e) {
      var me = this;

      if (me.isEnabled(el)) {
        // As usual, IE is always a PITA. Give em some love.
        var fr = [];
        _db.forEach(me.range(0, me.column(el), 1), function () {
          fr.push(['1fr']);
        });

        // -ms-grid-columns: 1fr 1fr 1fr 1fr;
        // grid-template-columns: repeat(4, 1fr);
        el.style.msGridColumns = fr.join(' ');
        el.style.gridTemplateColumns = 'repeat(' + me.column(el) + ', 1fr)';
      }
    },

    updateAttribute: function (box, item, el, e) {
      box.setAttribute('data-gs-width', item[2]);
      box.setAttribute('data-gs-height', item[3]);
    },

    init: function (el) {
      var me = this;

      me.$el = el;

      me.prepare(el);
      me.buildOut(el);
      me.cleanUp(el);
    }

  });

  /**
   * Attaches behavior to HTML element identified by .gridstack--native.
   *
   * @type {Drupal~behavior}
   */
  Drupal.behaviors.gridStackNative = {
    attach: function (context) {
      if ('length' in context) {
        context = context[0];
      }

      var me = Drupal.gridstack.native;
      var elms = context.querySelectorAll('.gridstack--native:not(.is-gn-on)');
      if (elms.length > 0) {
        _db.once(_db.forEach(elms, me.init.bind(me), context));
      }
    }
  };

})(Drupal, dBlazy);
